@@ -24,8 +24,10 @@ use strict;
 use warnings;
 use autodie;
 
-use constant NUMPY_STRING => 'module compiled against ABI version %x'
-  . ' but this version of numpy is %x';
+use constant NUMPY_REGEX => qr/
+    \Qmodule compiled against ABI version \E (?:0x)?%x
+    \Q but this version of numpy is \E (?:0x)?%x
+/xo;
 
 # These are the ones file(1) looks for.  The ".zdebug_info" being the
 # compressed version of .debug_info.
@@ -40,7 +42,6 @@ use Lintian::Check qw(check_spelling spelling_tag_emitter);
 use Lintian::Data;
 use Lintian::Relation qw(:constants);
 use Lintian::Tags qw(tag);
-use Lintian::Output qw(debug_msg);
 use Lintian::Util qw(fail slurp_entire_file strip);
 
 my $ARCH_REGEX = Lintian::Data->new('binaries/arch-regex', qr/\s*\~\~/o,
@@ -104,7 +105,7 @@ our $ARCH_32_REGEX;
 sub run {
     my ($pkg, $type, $info, $proc, $group) = @_;
 
-    my $madir;
+    my ($madir, %directories, $built_with_golang, %SONAME);
     my $gnu_triplet_re;
     my $ruby_triplet_re;
     my $needs_libc = '';
@@ -118,8 +119,6 @@ sub run {
     my $has_php_ext = 0;
     my $uses_numpy_c_abi = 0;
 
-    my %SONAME;
-
     my $arch = $info->field('architecture', '');
     my $multiarch = $info->field('multi-arch', 'no');
     my $srcpkg = $proc->pkg_src;
@@ -128,14 +127,17 @@ sub run {
         my $objdump = $info->objdump_info->{$file};
         my $has_lfs;
         my $is_profiled = 0;
+        # $file can be an object inside a static lib.  These do
+        # not appear in the output of our file_info collection.
+        my $file_info = $info->file_info($file) // '';
 
         # The LFS check only works reliably for ELF files due to the
         # architecture regex.
-        if ($objdump->{'ELF'}) {
+        if ($file_info =~ m/^[^,]*\bELF\b/o) {
             # Only 32bit ELF binaries can lack LFS.
             $ARCH_32_REGEX = $ARCH_REGEX->value('32')
               unless defined $ARCH_32_REGEX;
-            $has_lfs = 1 unless $info->file_info($file) =~ m/$ARCH_32_REGEX/o;
+            $has_lfs = 1 unless $file_info =~ m/$ARCH_32_REGEX/o;
             # We don't care if it is a debug file
             $has_lfs = 1 if $file =~ m,^usr/lib/debug/,;
         }
@@ -247,16 +249,7 @@ sub run {
     tag 'package-name-doesnt-match-sonames', "@sonames"
       if @sonames && !$match_found;
 
-    my %directories;
-    for my $file ($info->sorted_index) {
-        my $name = $file->name;
-        next unless $file->is_dir || $file->is_symlink;
-        $name =~ s,/\z,,;
-        $directories{"/$name"}++;
-    }
-
-    my $src = $group->get_source_processable();
-    my $built_with_golang;
+    my $src = $group->get_source_processable;
     if (defined($src)) {
         $built_with_golang
           = $src->info->relation('build-depends')->implies('golang-go');
@@ -264,8 +257,7 @@ sub run {
 
     # process all files in package
     foreach my $file ($info->sorted_index) {
-        my $fname = $file->name;
-        my ($fileinfo, $objdump);
+        my ($fileinfo, $objdump, $fname);
 
         next if not $file->is_file;
 
@@ -281,6 +273,7 @@ sub run {
             tag 'arch-independent-package-contains-binary-or-object',$file;
         }
 
+        $fname = $file->name;
         if ($fname =~ m,^etc/,) {
             tag 'binary-in-etc', $file;
         }
@@ -366,11 +359,11 @@ sub run {
         };
         my $tag_emitter
           = spelling_tag_emitter('spelling-error-in-binary', $file);
-        check_spelling($strings, $exceptions, $tag_emitter);
+        check_spelling($strings, $exceptions, $tag_emitter, 0);
 
         # stripped?
         if ($fileinfo =~ m,\bnot stripped\b,o) {
-            # Is it an object file (which generally can not be
+            # Is it an object file (which generally cannot be
             # stripped), a kernel module, debugging symbols, or
             # perhaps a debugging package?
             unless ($fname =~ m,\.k?o$,
@@ -399,6 +392,16 @@ sub run {
 
         # rpath is disallowed, except in private directories
         if (exists $objdump->{RPATH}) {
+            if (not %directories) {
+                for my $file ($info->sorted_index) {
+                    my $name;
+                    next unless $file->is_dir || $file->is_symlink;
+                    $name = $file->name;
+                    $name =~ s,/\z,,;
+                    $directories{"/$name"}++;
+                }
+            }
+
             foreach my $rpath (
                 map {File::Spec->canonpath($_)}
                 keys %{$objdump->{RPATH}}
@@ -448,7 +451,7 @@ sub run {
             or(     $fname =~ m,usr/lib/python3/.+\.cpython-\d+([a-z]+)\.so$,
                 and $1 !~ /d/)
           ) {
-            if (index($strings, NUMPY_STRING) != -1) {
+            if (index($strings, 'numpy') > -1 and $strings =~ NUMPY_REGEX) {
                 $uses_numpy_c_abi = 1;
             }
         }
@@ -458,7 +461,7 @@ sub run {
         if ($fname
             =~ m,^usr/lib/debug/(?:lib\d*|s?bin|usr|opt|dev|emul|\.build-id)/,)
         {
-            if (scalar(@{ $objdump->{NEEDED} })) {
+            if (exists($objdump->{NEEDED})) {
                 tag 'debug-file-should-use-detached-symbols', $file;
             }
             tag 'debug-file-with-no-debug-symbols', $file
@@ -470,14 +473,14 @@ sub run {
 
         # Detached debugging symbols directly in /usr/lib/debug.
         if ($fname =~ m,^usr/lib/debug/[^/]+$,) {
-            unless (scalar(@{ $objdump->{NEEDED} })
+            unless (exists($objdump->{NEEDED})
                 || $fileinfo =~ m/statically linked/) {
                 tag 'debug-symbols-directly-in-usr-lib-debug', $file;
             }
         }
 
         # statically linked?
-        if (!scalar(@{ $objdump->{NEEDED} })) {
+        if (!exists($objdump->{NEEDED})) {
             if ($fileinfo =~ m/shared object/o) {
                 # Some exceptions: kernel modules, syslinux modules, detached
                 # debugging information and the dynamic loader (which itself
@@ -518,9 +521,11 @@ sub run {
         } else {
             my $no_libc = 1;
             my $is_shared = 0;
+            my @needed;
             $needs_depends_line = 1;
             $is_shared = 1 if index($fileinfo, 'shared object') != -1;
-            for my $lib (@{$objdump->{NEEDED}}) {
+            @needed = @{$objdump->{NEEDED}} if exists($objdump->{NEEDED});
+            for my $lib (@needed) {
                 if ($lib =~ /^libc\.so\.(\d+.*)/) {
                     $needs_libc = "libc$1";
                     $needs_libc_file = $fname unless $needs_libc_file;
@@ -609,7 +614,7 @@ sub run {
     if ($has_perl_lib) {
         # It is a virtual package, so no version is allowed and
         # alternatives probably does not make sense here either.
-        my $re = qr/^perlapi-[\d.]+(?:\s*\[[^\]]+\])?$/;
+        my $re = qr/^perlapi-[-\w.]+(?:\s*\[[^\]]+\])?$/;
         unless ($depends->matches($re, VISIT_OR_CLAUSE_FULL)) {
             tag 'missing-dependency-on-perlapi';
         }
@@ -644,7 +649,7 @@ sub run {
 sub tag_unneeded_sections {
     my ($tag, $file, $objdump) = @_;
     foreach my $sect ('.note', '.comment') {
-        if (exists $objdump->{'SH'}->{$sect}) {
+        if (exists $objdump->{'SH'}{$sect}) {
             tag $tag, "$file $sect";
         }
     }
