@@ -281,6 +281,7 @@ sub run {
         if ($first_timestamp) {
             my $dch_date = $entries[0]->Date;
             my ($weekday_declared, $date) = split(m/,\s*/, $dch_date, 2);
+            $date //= '';
             my ($tz, $weekday_actual);
 
             if ($date =~ m/[ ]+ ([^ ]+)\Z/xsm) {
@@ -290,7 +291,7 @@ sub run {
             if ($tz and $weekday_declared ne $weekday_actual) {
                 my $real_weekday = time2str('%A', $first_timestamp, $tz);
                 my $short_date = time2str('%Y-%m-%d', $first_timestamp, $tz);
-                tag 'debian-changelog-has-wrong-weekday',
+                tag 'debian-changelog-has-wrong-day-of-week',
                   "$short_date is a $real_weekday";
             }
         }
@@ -363,6 +364,9 @@ sub run {
         while ($changes =~ /(closes\s*(?:bug)?\#?\s?\d{6,})[^\w]/ig) {
             tag 'possible-missing-colon-in-closes', $1 if $1;
         }
+        if ($changes =~ m/(TEMP-\d{7}-[0-9a-fA-F]{6})/) {
+            tag 'changelog-references-temp-security-identifier', $1;
+        }
 
         # check for bad intended distribution
         if (
