@@ -692,11 +692,12 @@ sub find_cruft {
 
         # Find mentioning of usr/lib/perl5 inside the packaging
         if ($name =~ m{^debian/}) {
-            my $short = $entry->basename();
+            my $short = $entry->basename;
 
             # Skip symlinks and other nasty stuff as well as
             # debian/changelog
             if (   $short ne 'changelog'
+                && $name !~ m{^debian/patches/.*$}
                 && $entry->is_file
                 && $entry->is_open_ok) {
                 my $contents = $entry->file_contents;
@@ -882,9 +883,9 @@ sub full_text_check {
     # string to match in $block. Please emit license tags only once
     # per file
   BLOCK:
-    while (my $block = $sfd->readwindow()) {
+    while (my $block = $sfd->readwindow) {
         my ($cleanedblock, %matchedkeyword);
-        my $blocknumber = $sfd->blocknumber();
+        my $blocknumber = $sfd->blocknumber;
 
         # Check for non-distributable files - this
         # applies even to non-free, as we still need
@@ -946,6 +947,12 @@ sub _search_in_block0 {
                 return;
             }
         }
+        if($basename eq 'search_index.js') {
+            if($block =~ m/\A\s*var\s*search_index\s*=/xms) {
+                tag 'source-contains-prebuilt-pandoc-documentation', $dirname;
+                return;
+            }
+        }
         # false positive in dx package at least
         elsif($basename eq 'srchidx.js') {
             if($block =~ m/\A\s*profiles \s* = \s* new \s* Array\s*\(/xms) {
@@ -1011,28 +1018,13 @@ sub _linelength_test_maxlength {
     return (0,'',$block);
 }
 
-# try to detect non human source based on line length
-sub _linelength_test {
-    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
-    my $linelength = 0;
-    my $line;
-    my $nextblock;
-
-    ($linelength)= _linelength_test_maxlength($block,INSANE_LINE_LENGTH);
-    # first check if line >  INSANE_LINE_LENGTH that is likely minification
-    # avoid problem by recursive regex with longline
-    if($linelength) {
-        _warn_prebuilt_javascript($entry, $info, $name, $basename, $dirname,
-            $linelength,INSANE_LINE_LENGTH);
-        return;
-    }
-    # Now try to be more clever and work only on the 8192 character
-    # in order to avoid regexp recursion problems
-    my $strip = substr($block,0,8192);
-    # strip indention
-    $strip =~ s/^\s+//g;
+# strip C comment
+# warning block is at more 8192 char in order to be too slow
+# and in order to avoid regex recursion
+sub _strip_c_comments {
+    my ($block) = @_;
     # from perl faq strip comments
-    $strip =~ s{
+    $block =~ s{
                 # Strip /* */ comments
                 /\* [^*]*+ \*++ (?: [^/*][^*]*+\*++ ) */
                 # Strip // comments (C++ style)
@@ -1046,6 +1038,56 @@ sub _linelength_test {
                     | .[^/"'\\]*+
                    )
                }{defined $1 ? $1 : ""}xgse;
+    return $block;
+}
+
+# try to detect non human source based on line length
+sub _linelength_test {
+    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
+    my $linelength = 0;
+    my $line;
+    my $nextblock;
+
+    ($linelength)= _linelength_test_maxlength($block,INSANE_LINE_LENGTH);
+    # first check if line >  INSANE_LINE_LENGTH that is likely minification
+    # avoid problem by recursive regex with longline
+    if($linelength) {
+        tag 'insane-line-length-in-source-file', $name,
+          'line length is', int($linelength),
+          'characters (>'.INSANE_LINE_LENGTH.')';
+        # clean up jslint craps line
+        $block =~ s,^\s*/[*][^\n]*[*]/\s*$,,gm;
+        $block =~ s,^\s*//[^\n]*$,,gm;
+        $block =~ s/^\s+//gm;
+
+        # try to remove comments in first 8192 block (license...)
+        my $block8192 = substr($block,0,8192);
+        $block8192 = _strip_c_comments($block8192);
+        $block
+          = length($block) > 8192
+          ? $block8192.substr($block,8192)
+          : $block8192;
+
+        # strip empty line
+        $block =~ s/^\s*\n//mg;
+        # remove last \n
+        $block =~ s/\n\Z//m;
+
+        # retry insane line length test now: if insane length likely minified
+        ($linelength)= _linelength_test_maxlength($block,INSANE_LINE_LENGTH);
+
+        if($linelength) {
+            _warn_prebuilt_javascript($entry, $info, $name, $basename,
+                $dirname,$linelength,INSANE_LINE_LENGTH);
+            return;
+        }
+    }
+    # Now try to be more clever and work only on the 8192 character
+    # in order to avoid regexp recursion problems
+    my $strip = substr($block,0,8192);
+    # strip indention
+    $strip =~ s/^\s+//mg;
+    $strip = _strip_c_comments($block);
     # strip empty line
     $strip =~ s/^\s*\n//mg;
     # remove last \n
