@@ -23,6 +23,8 @@ use strict;
 use warnings;
 use autodie;
 
+use Text::Levenshtein qw(distance);
+
 use Lintian::Data;
 use Lintian::Relation;
 use Lintian::Tags qw(tag);
@@ -155,7 +157,8 @@ sub run {
                 $seen_python_helper = 1;
                 $seen_python3_helper = 1;
             }
-        } elsif (m,^include\s+/usr/share/cdbs/1/rules/debhelper.mk,) {
+        } elsif (m,^include\s+/usr/share/cdbs/1/rules/debhelper.mk,
+            or m,^include\s+/usr/share/R/debian/r-cran.mk,o) {
             $build_systems{'cdbs-with-debhelper.mk'} = 1;
             delete($build_systems{'cdbs-without-debhelper.mk'});
             $seencommand = 1;
@@ -175,13 +178,44 @@ sub run {
             $dhcompatvalue = $1;
             # one can export and then set the value:
             $level = $1 if ($level);
-        } elsif (/^override_dh_/) {
+        } elsif (/^override_(dh_[^:]+)/) {
             $needbuilddepends = 1;
-        } elsif (m,^include\s+/usr/share/cdbs/,
-            or m,^include\s+/usr/share/R/debian/r-cran.mk,o) {
+            my $dhcommand = $1;
+            if (not $dh_commands_depends->known($dhcommand)) {
+                # Unknown command, so check for likely misspellings
+                foreach my $x (sort $dh_commands_depends->all) {
+                    if (distance($dhcommand, $x) < 3) {
+                        tag 'typo-in-debhelper-override-target',
+                          "override_$dhcommand", '->', "override_$x",
+                          "(line $.)";
+                        last; # Only emit a single match
+                    }
+                }
+            }
+        } elsif (m,^include\s+/usr/share/cdbs/,) {
             $inclcdbs = 1;
             $build_systems{'cdbs-without-debhelper.mk'} = 1
               if not exists($build_systems{'cdbs-with-debhelper.mk'});
+        } elsif (
+            m{
+              ^include \s+
+                 /usr/share/(?:
+                   dh-php/pkg-pecl\.mk
+                  |blends-dev/rules
+                 )
+              }xsm
+          ) {
+            # All of these indirectly use dh.
+            $build_systems{'dh'} = 1;
+            delete($build_systems{'debhelper'});
+        } elsif (
+            m{
+              ^include \s+
+                 /usr/share/pkg-kde-tool/qt-kde-team/\d+/debian-qt-kde\.mk
+              }xsm
+          ) {
+            $build_systems{'dhmk'} = 1;
+            delete($build_systems{'debhelper'});
         }
     }
     close($rules_fd);
