@@ -948,7 +948,7 @@ sub run {
         tag 'build-depends-indep-without-arch-indep'
           if (defined $info->field('build-depends-indep')
             && $arch_indep_packages == 0);
-        tag 'build-depends-arch-without-arch-dependant-binary'
+        tag 'build-depends-arch-without-arch-dependent-binary'
           if (defined $info->field('build-depends-arch')
             && $arch_dep_packages == 0);
 
@@ -1138,8 +1138,8 @@ sub run {
             my $build_conflicts_all = $info->relation('build-conflicts-all');
             tag 'restriction-formula-without-versioned-dpkg-dev-dependency'
               unless ($build_all->implies('dpkg-dev (>= 1.17.14~)'));
-            # if the package uses debhelper then it must require and not
-            # conflict with version >= 9.20141010
+            # if the package uses debhelper then it must require
+            # version >= 9.20141010
             if ($build_all->implies('debhelper')) {
                 tag
 'restriction-formula-with-debhelper-without-debhelper-version'
@@ -1309,6 +1309,14 @@ sub run {
         }
     }
 
+    #---- Checksums
+
+    if ($type eq 'source') {
+        if (!$info->field('checksums-sha256')) {
+            tag 'no-strong-digests-in-dsc';
+        }
+    }
+
     #----- Field checks (without checking the value)
 
     for my $field (keys %{$info->field}) {
