@@ -205,7 +205,7 @@ sub _detect_embedded_libraries {
                       = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); });
 
                   READWINDOW:
-                    while (my $block = $sfd->readwindow()) {
+                    while (my $block = $sfd->readwindow) {
                         if ($block =~ m{$reinside}) {
                             $foundre = 1;
                             last READWINDOW;
@@ -262,7 +262,6 @@ sub run {
     my ($is_python, $is_perl, $has_binary_perl_file);
     my @nonbinary_perl_files_in_lib;
     my %linked_against_libvga;
-    my $py_support_nver;
     my @devhelp;
     my @devhelp_links;
 
@@ -291,7 +290,7 @@ sub run {
     }
 
     # find out which files are scripts
-    my %script = map {$_ => 1} (sort keys %{$info->scripts});
+    my %script = map {$_ => 1} (keys %{$info->scripts});
 
     # We only want to warn about these once.
     my $warned_debug_name = 0;
@@ -319,7 +318,7 @@ sub run {
             unless ($fname =~ m,/$,) {
                 # Skip if $file is outside /usr/share/doc/$pkg directory
                 if ($fname !~ m,^usr/share/doc/\Q$pkg\E,) {
-                    # - except if it is an lintian override.
+                    # - except if it is a lintian override.
                     next
                       if $fname =~ m{\A
                             usr/share/lintian/overrides/$ppkg(?:\.gz)?
@@ -518,8 +517,8 @@ sub run {
             elsif ($fname =~ m,^etc/pam.conf, and $pkg ne 'libpam-runtime') {
                 tag 'config-file-reserved', "$fname by libpam-runtime";
             }
-            #----------------- /etc/php5/conf.d
-            elsif ($fname =~ m,^etc/php5/conf.d/.+\.ini$,) {
+            #----------------- /etc/php/7.0/mods-available/*.ini
+            elsif ($fname =~ m,^etc/php/.*/mods-available/.+\.ini$,) {
                 if ($file->is_file) {
                     my $fd = $file->open;
                     while (<$fd>) {
@@ -648,7 +647,7 @@ sub run {
                 my $fd = $file->open(':raw');
                 my $sfd = Lintian::SlidingWindow->new($fd);
               BLOCK:
-                while (my $block = $sfd->readwindow()) {
+                while (my $block = $sfd->readwindow) {
                     # remove comment line
                     $block =~ s,\#\V*,,gsm;
                     # remove continuation line
@@ -715,7 +714,7 @@ sub run {
             # ---------------- .packlist files
             elsif ($fname =~ m,^usr/lib/perl.*/.packlist$,) {
                 tag 'package-installs-packlist', $file;
-            }elsif ($fname =~ m,^usr/lib/(?:[^/]+/)?perl5/.*\.(?:pl|pm)$,) {
+            }elsif ($fname =~ m,^usr/lib/(?:[^/]+/)?perl5/.*\.p[lm]$,) {
                 push @nonbinary_perl_files_in_lib, $file;
             }elsif ($fname =~ m,^usr/lib/(?:[^/]+/)?perl5/.*\.(?:bs|so)$,) {
                 $has_binary_perl_file = 1;
@@ -723,7 +722,7 @@ sub run {
            # ---------------- /usr/lib -- needs to go after the other usr/lib/*
             elsif ($fname =~ m,^usr/lib/,) {
                 if (    $type ne 'udeb'
-                    and $file =~ m,\.(?:bmp|gif|jpeg|jpg|png|tiff|xpm|xbm)$,
+                    and $file =~ m,\.(?:bmp|gif|jpe?g|png|tiff|x[pb]m)$,
                     and not defined $link) {
                     tag 'image-file-in-usr-lib', $file;
                 }
@@ -835,7 +834,7 @@ sub run {
                     # config file (read only one block)
                     my $fd = $file->open(':raw');
                     my $sfd = Lintian::SlidingWindow->new($fd);
-                    my $block = $sfd->readwindow();
+                    my $block = $sfd->readwindow;
                     # some common stuff found in config file
                     if (
                         $block
@@ -843,16 +842,15 @@ sub run {
                             or index($block,'/include/') > -1
                             or index($block,'pkg-config')  > -1)
                       ) {
-                        # ok old config style script tag it
                         tag 'old-style-config-script',$file;
-                        my $multiarch = $info->field('multi-arch','');
+                        my $multiarch = $info->field('multi-arch', 'no');
                         # could be ok but only if multi-arch: no
                         if($multiarch ne 'no' or $arch eq 'all') {
                             # check multi-arch path
                             foreach my $archs ($MULTIARCH_DIRS->all) {
                                 my $madir= $MULTIARCH_DIRS->value($archs);
                                 if ($block =~ m{\W\Q$madir\E(\W|$)}xms){
-                             # allow files to begin by triplet if it match arch
+                         # allow files to begin with triplet if it matches arch
                                     if($file->basename =~ m{^\Q$madir\E}xms) {
                                         next;
                                     }
@@ -1097,9 +1095,9 @@ sub run {
         # ---------------- .egg (python egg files)
         if (
             $fname =~ m,\.egg$,o
-            && (   $fname =~ m,usr/lib/python\d+(?:\.\d+/),o
-                || $fname =~ m,usr/lib/pyshared,o
-                || $fname =~ m,usr/share/,o)
+            && (   $fname =~ m,^usr/lib/python\d+(?:\.\d+/),o
+                || $fname =~ m,^usr/lib/pyshared,o
+                || $fname =~ m,^usr/share/,o)
           ) {
             tag 'package-installs-python-egg', $file;
         }
@@ -1185,7 +1183,7 @@ sub run {
             #
             # Another exception is made for .html and .php because
             # preserving working links is more important than saving
-            # some bytes, and because a package had a HTML form for
+            # some bytes, and because a package had an HTML form for
             # licenses called like that.  Another exception is made
             # for various picture formats since those are likely to
             # just be simply pictures.
@@ -1201,8 +1199,8 @@ sub run {
             and ($operm & 0111) == 0
             and not $fname =~ m{ \. (?:
                   # Common "non-license" file extensions...
-                   el|[ch]|py|cc|pl|pm|hi|p_hi|html|php|rb|xpm
-                  |png|jpe?g|gif|svg|dtd|ui|pc|mk|lisp
+                   el|[ch]|cc|p[ylmc]|[hu]i|p_hi|html|php|rb|xpm
+                     |png|jpe?g|gif|svg|dtd|mk|lisp
                ) \Z}xsm
             and not $fname=~ m,^usr/share/zope/Products/.*\.(?:dtml|pt|cpt)$,
             and not $fname =~ m,/under\S+License\.docbook$,
@@ -1270,14 +1268,6 @@ sub run {
             tag 'file-should-not-be-compressed', $file;
         }
 
-        # ---------------- pyshared-data
-        if ($fname=~ m,^usr/share/python-support/$ppkg\.(?:public|private)$,){
-            $py_support_nver = '(>= 0.90)';
-        } elsif ($fname =~ m,^usr/share/python-support/\S+,o
-            && !$py_support_nver){
-            $py_support_nver = '';
-        }
-
         # ---------------- python file locations
         #  - The python people kindly provided the following table.
         # good:
@@ -1382,14 +1372,14 @@ sub run {
             if (
                 $fname =~ m{\A
                            (?:usr/)?(?:s?bin|games)/[^/]+\.
-                           (?:pl|sh|py|php|rb|tcl|bsh|csh|tcl)
+                           (?:p[ly]|php|rb|[bc]?sh|tcl)
                          \Z}xsm
               ) {
                 tag 'script-with-language-extension', $file;
             }
 
             # ---------------- Devel files for Windows
-            if (    $fname =~ m,/.+\.(?:vcproj|sln|dsp|dsw)(?:\.gz)?$,
+            if (    $fname =~ m,/.+\.(?:vcproj|sln|ds[pw])(?:\.gz)?$,
                 and $fname !~ m,^usr/share/doc/,) {
                 tag 'windows-devel-file-in-package', $file;
             }
@@ -1547,7 +1537,7 @@ sub run {
                     }
                     close($fd);
                     if ($mtime != 0) {
-                        if ($isma_same && $file !~ m/\Q$arch\E/o) {
+                        if ($isma_same && $file !~ m/\Q$arch\E/) {
                             tag 'gzip-file-is-not-multi-arch-same-safe',$file;
                         } else {
                             # see https://bugs.debian.org/762105
@@ -1863,33 +1853,19 @@ sub run {
           unless $type eq 'udeb';
     }
 
-    # python-support check
-    if (defined($py_support_nver) && $pkg ne 'python-support'){
-        # Okay - package installs something to /usr/share/python-support/
-        # $py_support_nver is either the empty string or a version
-        # describing what we need.
-        #
-        # We also skip debug packages since they are okay as long as
-        # foo-dbg depends on foo (= $version) and foo has its dependency
-        # correct.
-        my $dep = $info->relation('depends');
-        tag 'missing-dependency-on-python-support',
-          "python-support $py_support_nver"
-          unless ($pkg =~ m/-dbg$/
-            || $dep->implies("python-support $py_support_nver"));
-    }
-
     # Check for section games but nothing in /usr/games.  Check for
     # any binary to save ourselves from game-data false positives:
     my $games = dir_counts($info, 'usr/games/');
     my $other = dir_counts($info, 'bin/') + dir_counts($info, 'usr/bin/');
-    if ($pkg_section =~ m,games$, and $games == 0 and $other > 0) {
-        tag 'package-section-games-but-contains-no-game';
-    }
-    if ($pkg_section =~ m,games$, and $games > 0 and $other > 0) {
-        tag 'package-section-games-but-has-usr-bin';
-    }
-    if ($pkg_section !~ m,games$, and $games > 0 and $other == 0) {
+    if ($other) {
+        if ($pkg_section =~ m,games$,) {
+            if ($games) {
+                tag 'package-section-games-but-has-usr-bin';
+            } else {
+                tag 'package-section-games-but-contains-no-game';
+            }
+        }
+    } elsif ($games > 0 and $pkg_section !~ m,games$,) {
         tag 'games-package-should-be-section-games';
     }
 
@@ -1901,9 +1877,6 @@ sub run {
     # Empty Perl directories are an ExtUtils::MakeMaker artifact that
     # will be fixed in Perl 5.10, and people can cause more problems
     # by trying to fix it, so just ignore them.
-    #
-    # python-support needs a directory for each package even it might
-    # be empty
     if ($pkg ne 'base-files') {
         foreach my $dir ($info->sorted_index) {
             next if not $dir->is_dir;
@@ -1911,8 +1884,7 @@ sub run {
             next if ($dirname =~ m{^var/} or $dirname =~ m{^etc/});
             if (scalar($dir->children) == 0) {
                 if (    $dirname !~ m;^usr/lib/(?:[^/]+/)?perl5/$;
-                    and $dirname ne 'usr/share/perl5/'
-                    and $dirname !~ m;^usr/share/python-support/;) {
+                    and $dirname ne 'usr/share/perl5/') {
                     tag 'package-contains-empty-directory', $dirname;
                 }
             }
@@ -2108,7 +2080,7 @@ sub detect_privacy_breach {
 
     my $sfd = Lintian::SlidingWindow->new($fd,sub { $_=lc($_); },BLOCKSIZE);
 
-    while (my $block = $sfd->readwindow()) {
+    while (my $block = $sfd->readwindow) {
         # try generic fragment tagging
         foreach my $keyword ($PRIVACY_BREAKER_FRAGMENTS->all) {
             if(index($block,$keyword) > -1) {
