@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# Lintian reporting harness -- Run lintian against a archive mirror
+# Lintian reporting harness -- Run lintian against an archive mirror
 #
 # Copyright (C) 2015 Niels Thykier
 #
@@ -29,7 +29,7 @@ use autodie;
 
 use constant BACKLOG_PROCESSING_TIME_LIMIT => 4 * 3600; # 4hours
 
-use Fcntl qw(F_GETFD F_SETFD FD_CLOEXEC);
+use Fcntl qw(F_GETFD F_SETFD FD_CLOEXEC SEEK_END);
 use File::Basename qw(basename);
 use File::Temp qw(tempfile);
 use Getopt::Long();
@@ -109,7 +109,7 @@ sub prepare_lintian_environment_and_cmdline {
     my $frontend = 'lintian';
     my $eoa_marker_index = first_index { $_ eq '--' } @ARGV;
     my $logs_dir = $OPT{'lintian-log-dir'};
-    my @overridable_args = (qw(-EIL +pedantic --show-overrides));
+    my @overridable_args = (qw(-EL +>=classification --show-overrides));
     my @args = (
         qw(--verbose), # We rely on this for filtering the log
         qw(--exp-output=format=fullewi --packages-from-file -),
@@ -220,7 +220,8 @@ sub process_worklist {
         my (@work_splice, @completed, %processed);
         my ($lintpipe, $lint_stdin, $status_fd, $lint_status_out);
 
-        if (time() >= $start_time + BACKLOG_PROCESSING_TIME_LIMIT) {
+        # Bail if there is less than 5 minutes left
+        if (time() >= $start_time + BACKLOG_PROCESSING_TIME_LIMIT - 300) {
             log_msg('No more time for processing backlogs');
             $exit_code = 2;
             last;
@@ -294,26 +295,58 @@ sub process_worklist {
                 my $path = $members->{$member_id}{'path'};
                 print {$lintpipe} "$path\n";
             }
-            #print {$lintpipe} "!query: GROUP:$group_id\n";
         }
         close($lintpipe);
 
-        # Listen to status updates from lintian
-        while (my $line = <$status_fd>) {
-            chomp($line);
-            if ($line =~ s/^complete ([^ ]+) \(([^\)]+)\)$//) {
-                my ($group_id, $runtime) = ($1, $2);
-                push(@completed, $group_id);
-                $processed{$group_id} = 1;
-                log_msg("  [lintian] processed $group_id"
-                      . " successfully (time: $runtime)");
-            } elsif ($line =~ s/^error ([^ ]+) \(([^\)]+)\)$//) {
-                my ($group_id, $runtime) = ($1, $2);
-                log_msg(
-                    "  [lintian] error processing $group_id (time: $runtime)");
-                $processed{$group_id} = 1;
+        eval {
+            my $time_limit
+              = $start_time + BACKLOG_PROCESSING_TIME_LIMIT - time();
+            my $count = 0;
+            my $sig_handler = sub {
+                my ($signal_name) = @_;
+                $count++;
+                if ($count < 3) {
+                    log_msg("Received SIG${signal_name}, "
+                          . "sending SIGTERM to $pid [${count}/3]");
+                    kill('TERM', $pid);
+                    if ($signal_name eq 'ALRM') {
+                        log_msg(
+                            'Scheduling another alarm in 5 minutes from now...'
+                        );
+                        alarm(300);
+                    }
+                } else {
+                    log_msg("Received SIG${signal_name} as the third one, "
+                          . "sending SIGKILL to $pid");
+                    log_msg('You may have to clean up some '
+                          . 'temporary directories manually');
+                    kill('KILL', $pid);
+                }
+            };
+            local $SIG{'TERM'} = $sig_handler;
+            local $SIG{'INT'} = $sig_handler;
+            local $SIG{'ALRM'} = $sig_handler;
+
+            alarm($time_limit);
+
+            # Listen to status updates from lintian
+            while (my $line = <$status_fd>) {
+                chomp($line);
+                if ($line =~ m/^complete ([^ ]+) \(([^\)]+)\)$/) {
+                    my ($group_id, $runtime) = ($1, $2);
+                    push(@completed, $group_id);
+                    $processed{$group_id} = 1;
+                    log_msg("  [lintian] processed $group_id"
+                          . " successfully (time: $runtime)");
+                } elsif ($line =~ m/^error ([^ ]+) \(([^\)]+)\)$/) {
+                    my ($group_id, $runtime) = ($1, $2);
+                    log_msg("  [lintian] error processing $group_id "
+                          . "(time: $runtime)");
+                    $processed{$group_id} = 1;
+                }
             }
-        }
+            alarm(0);
+        };
         close($status_fd);
 
         # Wait for lintian to terminate
@@ -337,6 +370,7 @@ sub process_worklist {
             log_msg('Lintian finished successfully');
         }
         my $filter = generate_log_filter($state, \%processed);
+        seek($nfd, 0, SEEK_END);
         update_lintian_log($filter, $nfd, $new_lintian_log);
 
         log_msg('Updating harness state cache');
@@ -403,7 +437,7 @@ sub update_lintian_log {
             }
         }
         close($input);
-
+        close($new_fd);
         rename($tmp_path, $lintian_log);
     };
     if (my $err = $@) {
