@@ -97,6 +97,7 @@ use Digest::SHA;
 use Encode ();
 use File::Temp qw(tempfile);
 use FileHandle;
+use List::Util qw(shuffle);
 use Scalar::Util qw(openhandle);
 
 use Lintian::Command qw(spawn safe_qx);
@@ -361,10 +362,10 @@ An invalid or malformed PGP header appears.
 
 Two "BEGIN PGP MESSAGE" headers appears in the same message.
 
-=item End of file but expected a "END PGP SIGNATURE" header
+=item End of file but expected an "END PGP SIGNATURE" header
 
 The file ended after a "BEGIN PGP SIGNATURE" header without being
-followed by a "END PGP SIGNATURE".
+followed by an "END PGP SIGNATURE".
 
 =item PGP MESSAGE header must be first content if present
 
@@ -440,7 +441,7 @@ sub visit_dpkg_paragraph {
             # message format identical.
             die join(q{ },
                 "syntax error at line $.:",
-                qq{End of file but expected a "END PGP SIGNATURE" header\n})
+                qq{End of file but expected an "END PGP SIGNATURE" header\n})
               unless $saw_end;
         }
         # other pgp control?
@@ -709,7 +710,7 @@ L</parse_dpkg_control> do.  It can also emit:
         # control files.
         drain_pipe($opts->{pipe_out});
         close($opts->{pipe_out});
-        $opts->{harness}->finish();
+        $opts->{harness}->finish;
         return $data[0];
     }
 }
@@ -717,7 +718,7 @@ L</parse_dpkg_control> do.  It can also emit:
 =item get_dsc_control (DSCFILE)
 
 Convenience function for reading dsc files.  It will read the DSCFILE
-using L<read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
+using L</read_dpkg_control(FILE[, FLAGS[, LINES]])> and then return the
 first paragraph.  If the file has no paragraphs, C<undef> is returned
 instead.
 
@@ -1602,7 +1603,7 @@ sub save_state_cache {
             no autodie qw(unlink);
             unlink($tmp_path);
         }
-        die($@);
+        die($err);
     }
     return 1;
 }
@@ -1614,11 +1615,15 @@ harness state, return a list of group ids that are part of the
 backlog.  The list is sorted based on what version of Lintian
 processed the package.
 
+Note the result is by design not deterministic to reduce the
+risk of all large packages being in the same run (e.g. like
+gcc-5 + gcc-5-cross + gcc-6 + gcc-6-cross).
+
 =cut
 
 sub find_backlog {
     my ($lintian_version, $state) = @_;
-    my (@list, @sorted);
+    my (@backlog, %by_version);
     for my $group_id (keys(%{$state->{'groups'}})) {
         my $last_version = '0';
         my $group_data = $state->{'groups'}{$group_id};
@@ -1631,12 +1636,12 @@ sub find_backlog {
         }
         $is_out_of_date = 1
           if not versions_equal($last_version, $lintian_version);
-        push(@list, [$group_id, $last_version]) if $is_out_of_date;
+        push(@{$by_version{$last_version}}, $group_id) if $is_out_of_date;
     }
-    @sorted = map { $_->[0] }
-      sort { versions_comparator($a->[1], $b->[1]) || $a->[0] cmp $b->[0] }
-      @list;
-    return @sorted;
+    for my $v (sort(versions_comparator keys(%by_version))) {
+        push(@backlog, shuffle(@{$by_version{$v}}));
+    }
+    return @backlog;
 }
 
 =item untaint(VALUE)
